// timecombobox.cpp : implementation file
//

#include "stdafx.h"
#include "timecombobox.h"
#include "timehelper.h"
#include "misc.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CTimeComboBox

CTimeComboBox::CTimeComboBox(DWORD dwStyle) : m_dwStyle(dwStyle)
{
}

CTimeComboBox::~CTimeComboBox()
{
}


BEGIN_MESSAGE_MAP(CTimeComboBox, CComboBox)
	//{{AFX_MSG_MAP(CTimeComboBox)
	ON_WM_CREATE()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CTimeComboBox message handlers

void CTimeComboBox::PreSubclassWindow() 
{
	BuildCombo();
	
	// hook the edit ctrl so we can convert '.' and ',' to ':'
	CWnd* pEdit = GetDlgItem(1001);

	if (pEdit)
		ScHookWindow(pEdit->GetSafeHwnd());
	
	CComboBox::PreSubclassWindow();
}

int CTimeComboBox::OnCreate(LPCREATESTRUCT lpCreateStruct) 
{
	if (CComboBox::OnCreate(lpCreateStruct) == -1)
		return -1;
	
	BuildCombo();

	// hook the edit ctrl so we can convert '.' and ',' to ':'
	CWnd* pEdit = GetDlgItem(1001);

	if (pEdit)
		ScHookWindow(pEdit->GetSafeHwnd());
	
	return 0;
}

void CTimeComboBox::BuildCombo()
{
	if (GetCount() == 0)
	{
		for (int nHour = 0; nHour < 24; nHour++)
		{
			if ((m_dwStyle & TCB_NOTIME) && nHour == 0)
				AddString(""); // empty item meaning 'no time'
			else
				AddString(CTimeHelper::Format24HourTime(nHour, 0));

			if (m_dwStyle & TCB_HALFHOURS)
				AddString(CTimeHelper::Format24HourTime(nHour, 30));
		}
	}
}

double CTimeComboBox::GetOleTime() const
{
	return (Get24HourTime() / 24.0);
}

BOOL CTimeComboBox::SetOleTime(double dTime)
{
	// truncate to extract the time only component if it has one
	dTime -= (int)dTime;

	if (dTime <= 0)
	{
		SetCurSel(0);
		return TRUE;
	}
	
	// else
	return Set24HourTime(dTime * 24.0);
}

double CTimeComboBox::Get24HourTime() const
{
	// is this a hack? I'm not sure
	// but we look at the current Windows message being handled and if it's a
	// combo sel change then we use the current selection else we use the
	// window text (it may be an edit change notification)
	const MSG* pMsg = CComboBox::GetCurrentMessage();

	if (pMsg->message == WM_COMMAND && pMsg->lParam == (LPARAM)GetSafeHwnd() &&
		HIWORD(pMsg->wParam) == CBN_SELCHANGE)
	{
		// since the items in the combo are ordered from 1am to 11pm
		// we can use the selection index as a direct link to the hour
		int nSel = GetCurSel();

		if (nSel <= 0) // 'no time'
			return 0;

		// else
		if (m_dwStyle & TCB_HALFHOURS)
			return (nSel * 0.5);
		else
			return nSel;
	}

	// else use window text
	double dTime = 0;
	CString sTime;
	GetWindowText(sTime);

	if (sTime.IsEmpty())
	{
		if (m_dwStyle & TCB_NOTIME)
			return 0; // midnight
		else
			return -1; // error
	}

	// look for a separator
	CString sSep = Misc::GetTimeSeparator();
	int nColon = sTime.Find(sSep);

	if (nColon != -1) // extract hours and minutes
		dTime = atof(sTime.Left(nColon)) + atof(sTime.Mid(nColon + sSep.GetLength())) / 60;
	else
		dTime = atof(sTime);

	// look for PM signifier
	if (!Misc::GetPM().IsEmpty() && dTime < 12)
	{
		sTime.MakeUpper();
		
		if (sTime.Find(Misc::GetPM()) != -1)
			dTime += 12;
	}

	return (dTime <= 0) ? 0 : dTime;
}

BOOL CTimeComboBox::Set24HourTime(double dTime)
{
	if (dTime < 0)
	{
		SetCurSel(0);
		return TRUE;
	}
	
	// else
	if (dTime >= 24.0)
		return FALSE;

	// add on half a minute to handle floating point inaccuracies
	dTime += (0.5 / (24 * 60));

	int nHour = (int)dTime;
	int nMin = (int)((dTime - nHour) * 60); // nearest 1 minute

	CString sTime = CTimeHelper::Format24HourTime(nHour, nMin);

	if (SelectString(-1, sTime) == CB_ERR)
		SetWindowText(sTime);

	return TRUE;
}

LRESULT CTimeComboBox::ScWindowProc(HWND hRealWnd, UINT message, WPARAM wParam, LPARAM lParam)
{
	switch (message)
	{
	case WM_CHAR:
		if (wParam == ',' || wParam == '.' && !Misc::GetTimeSeparator().IsEmpty())
		{
			CString sSep = Misc::GetTimeSeparator();
			return CSubclasser::ScWindowProc(hRealWnd, message, sSep[0], lParam);
		}
		break;
	}

	// else
	return ScDefault(hRealWnd);
}
