// TDLPrintDialog.cpp : implementation file
//

#include "stdafx.h"
#include "todolist.h"
#include "TDLPrintDialog.h"

#include "..\shared\enstring.h"
#include "..\shared\preferences.h"
#include "..\shared\filemisc.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CTDLPrintDialog dialog


CTDLPrintDialog::CTDLPrintDialog(LPCTSTR szTitle, BOOL bPreview, BOOL bShowSubtaskCheckbox, CWnd* pParent /*=NULL*/)
	: CDialog(IDD_PRINT_DIALOG, pParent), 
	m_bPreview(bPreview), 
	m_taskSel("Print", bShowSubtaskCheckbox),
	m_sTitle(szTitle), 
	m_eStylesheet(FES_COMBOSTYLEBTN, CEnString(IDS_XSLFILEFILTER))
{
	//{{AFX_DATA_INIT(CTDLPrintDialog)
	//}}AFX_DATA_INIT
	CPreferences prefs;

	m_bDate = prefs.GetProfileInt("Print", "WantDate", TRUE);
	m_sStylesheet = prefs.GetProfileString("Print", "Stylesheet");

	// check whether user has set stylesheet in prefs
	if (m_sStylesheet.IsEmpty() || prefs.GetProfileInt("Print", "DefaultStylesheet", FALSE))
	{
		CString sDefStylesheet = prefs.GetProfileString("Preferences", "PrintStylesheet");

		if (!sDefStylesheet.IsEmpty())
			m_sStylesheet = sDefStylesheet;
	}

	m_bUseStylesheet = (!m_sStylesheet.IsEmpty());
}


void CTDLPrintDialog::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CTDLPrintDialog)
	DDX_Control(pDX, IDC_STYLESHEET, m_eStylesheet);
	DDX_Text(pDX, IDC_STYLESHEET, m_sStylesheet);
	//}}AFX_DATA_MAP
	DDX_Text(pDX, IDC_PRINTTITLE, m_sTitle);
	DDX_Check(pDX, IDC_PRINTDATE, m_bDate);
	DDX_Check(pDX, IDC_USESTYLESHEET, m_bUseStylesheet);
}


BEGIN_MESSAGE_MAP(CTDLPrintDialog, CDialog)
	//{{AFX_MSG_MAP(CTDLPrintDialog)
	ON_EN_CHANGE(IDC_STYLESHEET, OnChangeStylesheet)
	//}}AFX_MSG_MAP
	ON_BN_CLICKED(IDC_USESTYLESHEET, OnUsestylesheet)
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CTDLPrintDialog message handlers

void CTDLPrintDialog::OnOK() 
{
	CDialog::OnOK();
	
	// save settings
	CPreferences prefs;

	prefs.WriteProfileString("Print", "Stylesheet", m_bUseStylesheet ? m_sStylesheet : "");

	// we store whether this is the same as the default print stylesheet
	// so we can update as it does
	CString sDefStylesheet = prefs.GetProfileString("Preferences", "PrintStylesheet");
	prefs.WriteProfileInt("Print", "DefaultStylesheet", (m_sStylesheet.CompareNoCase(sDefStylesheet) == 0));
}


BOOL CTDLPrintDialog::OnInitDialog() 
{
	CDialog::OnInitDialog();

    VERIFY(m_taskSel.Create(IDC_FRAME, this));

	BOOL bEnable = !m_bUseStylesheet || FileMisc::FileExists(FileMisc::GetFullPath(m_sStylesheet, TRUE));
	GetDlgItem(IDOK)->EnableWindow(bEnable);

	if (m_bPreview)
		SetWindowText(CEnString(IDS_PRINTDLG_PREVIEW_TITLE));
	else
		SetWindowText(CEnString(IDS_PRINTDLG_PRINT_TITLE));

	GetDlgItem(IDC_STYLESHEET)->EnableWindow(m_bUseStylesheet);

	// init the stylesheet folder to point to the resource folder
	CString sXslFolder = FileMisc::GetModuleFolder() + "Resources";
	m_eStylesheet.SetCurrentFolder(sXslFolder);
	
	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}

void CTDLPrintDialog::OnUsestylesheet() 
{
	UpdateData();

	GetDlgItem(IDC_STYLESHEET)->EnableWindow(m_bUseStylesheet); 

	BOOL bEnable = !m_bUseStylesheet || FileMisc::FileExists(FileMisc::GetFullPath(m_sStylesheet, TRUE));
	GetDlgItem(IDOK)->EnableWindow(bEnable);
}

void CTDLPrintDialog::OnChangeStylesheet() 
{
	UpdateData();

	BOOL bEnable = !m_bUseStylesheet || FileMisc::FileExists(FileMisc::GetFullPath(m_sStylesheet, TRUE));
	GetDlgItem(IDOK)->EnableWindow(bEnable);
}

CString CTDLPrintDialog::GetStylesheet() const 
{ 
	if (m_bUseStylesheet)
		return FileMisc::GetFullPath(m_sStylesheet, TRUE);
	
	// else
	return ""; 
}
