// ToDoCtrlReminders.cpp : implementation file
//

#include "stdafx.h"
#include "todolist.h"
#include "ToDoCtrlReminders.h"
#include "filteredToDoCtrl.h"
#include "..\shared\preferences.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CToDoCtrlReminders

CToDoCtrlReminders::CToDoCtrlReminders() : m_pWndNotify(NULL)
{
}

CToDoCtrlReminders::~CToDoCtrlReminders()
{
}


BEGIN_MESSAGE_MAP(CToDoCtrlReminders, CWnd)
	//{{AFX_MSG_MAP(CToDoCtrlReminders)
	ON_WM_TIMER()
	ON_WM_ACTIVATE()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()


/////////////////////////////////////////////////////////////////////////////
// CToDoCtrlReminders message handlers

BOOL CToDoCtrlReminders::Initialize(CWnd* pNotify)
{
	ASSERT_VALID(pNotify);

	if (!pNotify || !pNotify->GetSafeHwnd())
		return FALSE;

	m_pWndNotify = pNotify;

	// create ourselves so that we can receive timer messages
	if (!Create("STATIC", "", WS_CHILD, CRect (0, 0, 0, 0), pNotify, 0xffff))
	{
		m_pWndNotify = NULL;
		return FALSE;
	}

	// else
	return TRUE;
}

void CToDoCtrlReminders::AddToDoCtrl(const CFilteredToDoCtrl& tdc)
{
	LoadReminders(tdc);
}

void CToDoCtrlReminders::CloseToDoCtrl(const CFilteredToDoCtrl& tdc)
{
	SaveAndRemoveReminders(tdc);
}

void CToDoCtrlReminders::SetReminder(const TDCREMINDER& rem)
{
	ASSERT (m_pWndNotify);

	int nRem = FindReminder(rem);
	TDCREMINDER temp = rem; // to get around constness

	if (nRem != -1) // already exists
		m_aReminders[nRem] = rem; // replace
	else
		m_aReminders.Add(temp);

	// start time if we have any reminders
	if (m_aReminders.GetSize())
#ifdef _DEBUG
		SetTimer(1, 10000, NULL);
#else
		SetTimer(1, 60000, NULL); // every minute
#endif
}

void CToDoCtrlReminders::RemoveReminder(const TDCREMINDER& rem)
{
	RemoveReminder(rem.dwTaskID, rem.pTDC);
}

void CToDoCtrlReminders::RemoveReminder(DWORD dwTaskID, const CFilteredToDoCtrl* pTDC)
{
	ASSERT (m_pWndNotify);

	int nRem = FindReminder(dwTaskID, pTDC);

	if (nRem != -1) // already exists
	{
		m_aReminders.RemoveAt(nRem);

		// kill time if we have no reminders
		if (m_aReminders.GetSize() == 0)
			KillTimer(1);
	}
}

BOOL CToDoCtrlReminders::GetReminder(int nRem, TDCREMINDER& rem) const
{
	if (nRem < 0 || nRem >= m_aReminders.GetSize())
		return FALSE;

	rem = m_aReminders[nRem];
	return TRUE;
}

int CToDoCtrlReminders::FindReminder(const TDCREMINDER& rem) const
{
	return FindReminder(rem.dwTaskID, rem.pTDC);
}

int CToDoCtrlReminders::FindReminder(DWORD dwTaskID, const CFilteredToDoCtrl* pTDC) const
{
	int nRem = m_aReminders.GetSize();

	while (nRem--)
	{
		TDCREMINDER rem = m_aReminders[nRem];

		if (dwTaskID == rem.dwTaskID && pTDC == rem.pTDC)
			return nRem;
	}

	return -1;
}

void CToDoCtrlReminders::SaveAndRemoveReminders(const CFilteredToDoCtrl& tdc)
{
	CPreferences prefs;
	int nRemCount = 0, nRem = m_aReminders.GetSize();
	CString sFileKey = "Reminders\\" + CPreferences::KeyFromFile(tdc.GetFilePath(), FALSE);

	while (nRem--)
	{
		TDCREMINDER rem = m_aReminders[nRem];

		if (rem.pTDC == &tdc)
		{
			CString sKey;
			sKey.Format("%s\\Reminder%d", sFileKey, nRem);

			// note: we don't save the snooze value, this gets reset each time
			prefs.WriteProfileInt(sKey, "TaskID", rem.dwTaskID);
			prefs.WriteProfileInt(sKey, "Relative", rem.bRelative);

			if (rem.bRelative)
			{
				prefs.WriteProfileDouble(sKey, "LeadIn", rem.dRelativeDaysLeadIn * 24 * 60); // save as minutes
				prefs.WriteProfileInt(sKey, "FromWhen", rem.nRelativeFromWhen);
			}
			else
				prefs.WriteProfileDouble(sKey, "AbsoluteDate", rem.dtAbsolute.m_dt);
			
			prefs.WriteProfileInt(sKey, "Enabled", rem.bEnabled);
			prefs.WriteProfileString(sKey, "SoundFile", rem.sSoundFile);

			nRemCount++;
			m_aReminders.RemoveAt(nRem);
		}
	}

	prefs.WriteProfileInt(sFileKey, "NumReminders", nRemCount);

	// kill timer if no reminders
	if (m_aReminders.GetSize() == 0)
		KillTimer(1);
}

void CToDoCtrlReminders::LoadReminders(const CFilteredToDoCtrl& tdc)
{
	CPreferences prefs;
	CString sFileKey = "Reminders\\" + CPreferences::KeyFromFile(tdc.GetFilePath(), FALSE);
	int nRemCount = prefs.GetProfileInt(sFileKey, "NumReminders");

	for (int nRem = 0; nRem < nRemCount; nRem++)
	{
		CString sKey;
		sKey.Format("%s\\Reminder%d", sFileKey, nRem);

		TDCREMINDER rem;
		rem.dwTaskID = prefs.GetProfileInt(sKey, "TaskID");
		rem.bRelative = prefs.GetProfileInt(sKey, "Relative");

		if (rem.bRelative)
		{
			rem.dRelativeDaysLeadIn = prefs.GetProfileDouble(sKey, "LeadIn") / (24 * 60);
			rem.nRelativeFromWhen = (TDC_REMINDER)prefs.GetProfileInt(sKey, "FromWhen");
		}
		else
			rem.dtAbsolute = prefs.GetProfileDouble(sKey, "AbsoluteDate");

		rem.bEnabled = prefs.GetProfileInt(sKey, "Enabled");
		rem.sSoundFile = prefs.GetProfileString(sKey, "SoundFile");
		rem.pTDC = &tdc;

		m_aReminders.Add(rem);
	}

	// start timer if some reminders
	if (m_aReminders.GetSize())
#ifdef _DEBUG
		SetTimer(1, 10000, NULL);
#else
		SetTimer(1, 60000, NULL); // every minute
#endif
}

void CToDoCtrlReminders::OnTimer(UINT nIDEvent) 
{
	// iterate all the reminders looking for matches
	int nRem = m_aReminders.GetSize();

	while (nRem--)
	{
		TDCREMINDER rem = m_aReminders[nRem];

		// check for disabled reminders
		if (!rem.bEnabled)
			continue;

		// check for completed tasks
		if (rem.pTDC->IsTaskDone(rem.dwTaskID, TDCCHECKALL))
		{
			m_aReminders.RemoveAt(nRem);
			continue;
		}

		// else
		BOOL bNotify = FALSE;
		COleDateTime dateNow = COleDateTime::GetCurrentTime();

		if (rem.bRelative)
		{
			COleDateTime dateRem;

			if (rem.nRelativeFromWhen == TDCR_DUEDATE)
			dateRem = rem.pTDC->GetTaskDate(rem.dwTaskID, TDCD_DUE);
		else // start date
			dateRem = rem.pTDC->GetTaskDate(rem.dwTaskID, TDCD_START);

			bNotify = (dateRem > 0 && (dateNow.m_dt + rem.dRelativeDaysLeadIn - rem.dDaysSnooze > dateRem.m_dt));
		}
		else // absolute
		{
			bNotify = (dateNow.m_dt - rem.dDaysSnooze > rem.dtAbsolute);
		}

		if (bNotify)
		{
			ASSERT (m_pWndNotify);

			if (m_pWndNotify->SendMessage(WM_TD_REMINDER, 0, (LPARAM)&rem) == 0L)
				m_aReminders[nRem] = rem;
			else
			{
				m_aReminders.RemoveAt(nRem);
				rem.pTDC->RedrawReminders();
			}
		}
	}
	
	CWnd::OnTimer(nIDEvent);
}
